package au.net.netstorm.boost.bullet.clobber.read.lex;

import java.util.Arrays;
import java.util.List;
import au.net.netstorm.boost.bullet.clobber.core.ClobberTestCase;
import static au.net.netstorm.boost.bullet.clobber.read.lex.Type.ATOM;
import static au.net.netstorm.boost.bullet.clobber.read.lex.Type.CLOSE;
import static au.net.netstorm.boost.bullet.clobber.read.lex.Type.OPEN;
import static au.net.netstorm.boost.bullet.clobber.read.lex.Type.SEPARATOR;
import au.net.netstorm.boost.sniper.marker.HasFixtures;
import au.net.netstorm.boost.sniper.marker.InjectableTest;
import au.net.netstorm.boost.spider.api.runtime.Nu;
import junit.framework.Assert;

public final class LexerModuleTest extends ClobberTestCase implements HasFixtures, InjectableTest {
    private Token topen;
    private Token tclose;
    private Token tatom;
    private Token tseparator;
    private Token tpretty;
    private Token tsingle;
    private Token tdouble;
    private Token tslasher;
    String open = "(";
    String close = ")";
    String atom = "atom";
    String unsingle = " sin gle\"s ";
    String singles = "'" + unsingle + "'";
    String undouble = " dou ble's ";
    String doubles = "\"" + undouble + "\"";
    String unslasher = "foo\\bar";
    String slasher = "'foo\\\\bar'";
    String separator = " ";
    String pretty = "\n";
    String simple = open + atom + close;
    String compound = open + atom + separator + atom + close;
    String nested = open + atom + separator + open + atom + separator + atom + close + close;
    String formatted = open + atom + pretty + atom + close;
    String stringsingle = open + singles + close;
    String stringdouble = open + doubles + close;
    String stringslash = open + slasher + close;
    Lexer subject;
    Nu nu;

    public void fixtures() {
        topen = nu.nu(Token.class, OPEN, open);
        tclose = nu.nu(Token.class, CLOSE, close);
        tatom = nu.nu(Token.class, ATOM, atom);
        tseparator = nu.nu(Token.class, SEPARATOR, separator);
        tpretty = nu.nu(Token.class, SEPARATOR, pretty);
        tsingle = nu.nu(Token.class, ATOM, unsingle);
        tdouble = nu.nu(Token.class, ATOM, undouble);
        tslasher = nu.nu(Token.class, ATOM, unslasher);
    }

    public void test() {
        check(open, topen);
        check(close, tclose);
        check(simple, topen, tatom, tclose);
        check(compound, topen, tatom, tseparator, tatom, tclose);
        check(nested, topen, tatom, tseparator, topen, tatom, tseparator, tatom, tclose, tclose);
        check(formatted, topen, tatom, tpretty, tatom, tclose);
        check(stringdouble, topen, tdouble, tclose);
        check(stringsingle, topen, tsingle, tclose);
        check(stringslash, topen, tslasher, tclose);
    }

    private void check(String text, Token... tokens) {
        List list = Arrays.asList(tokens);
        Tokens expected = nu.nu(Tokens.class, list);
        Tokens actual = subject.lex(text);
        Assert.assertEquals(expected, actual);
    }
}
