package au.net.netstorm.boost.bullet.incredibles.engine.layer;

import java.util.Map;
import au.net.netstorm.boost.gunge.goop.Nvp;
import au.net.netstorm.boost.gunge.layer.Layers;
import au.net.netstorm.boost.gunge.layer.Method;
import au.net.netstorm.boost.gunge.layer.Methods;
import au.net.netstorm.boost.gunge.layer.Stripper;
import au.net.netstorm.boost.spider.api.lifecycle.Constructable;

public class DefaultStructLayer implements StructLayer, Constructable {
    private IncredibleSuit suit;
    private Nvp[] nvps;
    Map<String, Nvp> lookup;
    Stripper stripper;
    Methods methods;
    Layers layers;

    public DefaultStructLayer(IncredibleSuit suit, Nvp[] nvps) {
        this.suit = suit;
        this.nvps = nvps;
    }

    public void constructor() {
        for (Nvp nvp : nvps) save(nvp);
    }

    public Object invoke(Method method, Object[] params) {
        if (shortcircuit(method, Object.class)) return shortcircuit(method, params);
        return values(method);
    }

    private boolean shortcircuit(Method method, Class cls) {
        return method.getDeclaringClass() == cls;
    }

    private Object shortcircuit(Method method, Object[] params) {
        if (methods.is(method, "equals")) return equal(params[0]);
        if (methods.is(method, "hashCode")) return suit.hashCode();
        if (methods.is(method, "toString")) return suit.toString();
        throw new UnsupportedOperationException("" + method);
    }

    private Object values(Method method) {
        String name = method.getName();
        if (!lookup.containsKey(name)) throw new UnsupportedOperationException("" + method);
        Nvp nvp = lookup.get(name);
        return nvp.value();
    }

    // OK IllegalType {
    private Object equal(Object param) {
        if (!layers.is(param)) return false;
        DefaultStructLayer other = stripper.strip(DefaultStructLayer.class, param);
        return suit.equals(other.suit);
    }
    // } OK IllegalType

    private void save(Nvp nvp) {
        String name = nvp.name();
        lookup.put(name, nvp);
    }
}
