package au.net.netstorm.boost.bullet.incredibles.engine.layer;

import java.lang.reflect.Method;
import au.net.netstorm.boost.bullet.incredibles.engine.core.IncredibleTypes;
import au.net.netstorm.boost.gunge.goop.Nvp;
import au.net.netstorm.boost.gunge.reflect.method.MethodInterrogator;
import au.net.netstorm.boost.gunge.type.Interface;

public class DefaultStructValidator implements StructValidator {
    MethodInterrogator methods;
    IncredibleTypes types;

    public void check(Interface iface, Nvp[] nvps) {
        if (!types.struct(iface)) throw new IllegalArgumentException("Expected a struct");
        methods(iface, nvps);
    }

    private void methods(Interface iface, Nvp[] nvps) {
        Class type = iface.v();
        Method[] methods = type.getMethods();
        for (Method method : methods) check(method, nvps);
    }

    private void check(Method method, Nvp[] nvps) {
        if (methods.parameterized(method)) barf("Params not allowed " + method);
        if (methods.primitive(method)) barf("Primitives not supported " + method);
        if (!hasField(method, nvps)) barf("No field for method " + method);
    }

    private boolean hasField(Method method, Nvp[] nvps) {
        for (Nvp nvp : nvps) {
            if (equal(nvp, method)) return true;
        }
        return false;
    }

    private boolean equal(Nvp nvp, Method method) {
        Object value = nvp.value();
        Class type = value.getClass();
        String name = nvp.name();
        return methods.matches(method, type, name);
    }

    private void barf(String msg) {
        throw new IllegalArgumentException(msg);
    }
}
