package au.net.netstorm.boost.bullet.log;

import static au.net.netstorm.boost.bullet.log.LogLevel.ERROR;
import static au.net.netstorm.boost.bullet.log.LogLevel.INFO;
import static au.net.netstorm.boost.bullet.log.LogLevel.TRACE;
import static au.net.netstorm.boost.bullet.log.LogLevel.WARN;
import au.net.netstorm.boost.sniper.core.LifecycleTestCase;
import au.net.netstorm.boost.sniper.marker.InjectableTest;
import au.net.netstorm.boost.sniper.marker.LazyFields;
import au.net.netstorm.boost.sniper.marker.OverlaysWeb;
import au.net.netstorm.boost.spider.api.builder.Spinneret;

public final class DelegatingLogAtomicTest extends LifecycleTestCase implements InjectableTest, LazyFields, OverlaysWeb {
    Log subject;
    LogEngine engineMock;
    Throwable tDummy;
    Object oDummy;

    public void overlay() {
        Spinneret spinneret = spider.resolve(Spinneret.class);
        spinneret.spin(LogWeb.class);
        wire.ref(engineMock).to(LogEngine.class, DelegatingLog.class);
    }

    public void testTraceObject() {
        expectLog(TRACE, oDummy);
        subject.trace(oDummy);
    }

    public void testTraceThrowable() {
        expectLog(TRACE, tDummy);
        subject.trace(tDummy);
    }

    public void testTraceObjectAndThrowable() {
        expectLog(TRACE, oDummy, tDummy);
        subject.trace(oDummy, tDummy);
    }

    public void testInfoObject() {
        expectLog(INFO, oDummy);
        subject.info(oDummy);
    }

    public void testInfoThrowable() {
        expectLog(INFO, tDummy);
        subject.info(tDummy);
    }

    public void testInfoObjectAndThrowable() {
        expectLog(INFO, oDummy, tDummy);
        subject.info(oDummy, tDummy);
    }

    public void testWarnObject() {
        expectLog(WARN, oDummy);
        subject.warn(oDummy);
    }

    public void testWarnThrowable() {
        expectLog(WARN, tDummy);
        subject.warn(tDummy);
    }

    public void testWarnObjectAndThrowable() {
        expectLog(WARN, oDummy, tDummy);
        subject.warn(oDummy, tDummy);
    }

    public void testErrorObject() {
        expectLog(ERROR, oDummy);
        subject.error(oDummy);
    }

    public void testErrorThrowable() {
        expectLog(ERROR, tDummy);
        subject.error(tDummy);
    }

    public void testErrorObjectAndThrowable() {
        expectLog(ERROR, oDummy, tDummy);
        subject.error(oDummy, tDummy);
    }

    private void expectLog(Object... expected) {
        expect.oneCall(engineMock, VOID, "log", expected);
    }
}
