package au.net.netstorm.boost.bullet.roughly;

import au.net.netstorm.boost.bullet.time.core.Duration;
import static au.net.netstorm.boost.bullet.time.core.TimeUnit.MONTHS;
import static au.net.netstorm.boost.bullet.time.type.TimeConstants.ONE_DAY;
import static au.net.netstorm.boost.bullet.time.type.TimeConstants.ONE_HOUR;
import static au.net.netstorm.boost.bullet.time.type.TimeConstants.ONE_MILLISECOND;
import static au.net.netstorm.boost.bullet.time.type.TimeConstants.ONE_MINUTE;
import static au.net.netstorm.boost.bullet.time.type.TimeConstants.ONE_SECOND;
import static au.net.netstorm.boost.bullet.time.type.TimeConstants.ONE_WEEK;
import static au.net.netstorm.boost.bullet.time.type.TimeConstants.ONE_YEAR;
import au.net.netstorm.boost.sniper.core.LifecycleTestCase;
import au.net.netstorm.boost.sniper.marker.InjectableTest;

public final class DefaultRoughlyAtomicTest extends LifecycleTestCase implements InjectableTest {
    private static final Long ONE_MONTH = MONTHS.millis;
    private static final Long SOME = 50L;
    Roughly subject;

    public void test() {
        seconds();
        minutes();
        hours();
        days();
        weeks();
        months();
        years();
    }

    private void seconds() {
        check("just happened", ONE_SECOND - ONE_MILLISECOND);
        check("1 second", ONE_SECOND);
        check("2 seconds", 2 * ONE_SECOND);
        check("2 seconds", 2 * ONE_SECOND + SOME);
        check("39 seconds", 39 * ONE_SECOND);
        check("almost a minute", 40 * ONE_SECOND);
    }

    private void minutes() {
        check("almost a minute", ONE_MINUTE - ONE_MILLISECOND);
        check("1 minute", ONE_MINUTE);
        check("2 minutes", 2 * ONE_MINUTE);
        check("2 minutes", 2 * ONE_MINUTE + SOME);
        check("19 minutes", 19 * ONE_MINUTE);
        check("half an hour", 20 * ONE_MINUTE);
        check("half an hour", 39 * ONE_MINUTE);
        check("almost an hour", 40 * ONE_MINUTE);
        check("almost an hour", 59 * ONE_MINUTE);
    }

    private void hours() {
        check("almost an hour", ONE_HOUR - ONE_MILLISECOND);
        check("1 hour", ONE_HOUR);
        check("2 hours", 2 * ONE_HOUR);
        check("half a day", 4 * ONE_HOUR);
        check("almost a day", 8 * ONE_HOUR);
    }

    private void days() {
        check("almost a day", ONE_DAY - ONE_MILLISECOND);
        check("1 day", ONE_DAY);
        check("2 days", 2 * ONE_DAY);
        check("4 days", 4 * ONE_DAY);
        check("almost a week", 5 * ONE_DAY);
    }

    private void weeks() {
        check("almost a week", ONE_WEEK - ONE_MILLISECOND);
        check("1 week", ONE_WEEK);
        check("2 weeks", 2 * ONE_WEEK);
        check("3 weeks", 3 * ONE_WEEK);
        check("almost a month", 4 * ONE_WEEK);
    }

    private void months() {
        check("almost a month", ONE_MONTH - ONE_MILLISECOND);
        check("1 month", ONE_MONTH);
        check("2 months", 2 * ONE_MONTH);
        check("half a year", 6 * ONE_MONTH);
        check("over half a year", 7 * ONE_MONTH);
        check("almost a year", 9 * ONE_MONTH);
    }

    private void years() {
        check("1 year", ONE_YEAR);
        check("2 years", 2 * ONE_YEAR);
        check("2 years", 2 * ONE_YEAR + SOME);
        check("3 years", 3 * ONE_YEAR);
        check("4 years", 4 * ONE_YEAR);
        check("5 years", 5 * ONE_YEAR);
        check("6 years", 6 * ONE_YEAR);
        check("99 years", 99 * ONE_YEAR);
    }

    private void check(String expected, Long duration) {
        Duration d = new Duration(duration);
        String actual = subject.is(d);
        assertEquals(expected, actual);
    }
}
