package au.net.netstorm.boost.bullet.scalpel.engine;

import au.net.netstorm.boost.gunge.layer.DefaultMethod;
import au.net.netstorm.boost.gunge.layer.Method;
import au.net.netstorm.boost.gunge.sledge.java.lang.EdgeClass;

final class DefaultMethodWarp implements MethodWarp {
    GroupUnedger groups;
    EdgeClass classer;
    Unedger unedger;

    public Method warp(Class realClass, Method edgeMethod) {
        String name = edgeMethod.getName();
        Class[] edgeParams = edgeMethod.getParameterTypes();
        Class[] realParams = groups.unedge(edgeParams);
        Method realMethod = unedge(realClass, name, realParams);
        validate(realMethod, edgeMethod);
        return realMethod;
    }

    private Method unedge(Class real, String name, Class[] realParams) {
        java.lang.reflect.Method method = classer.getMethod(real, name, realParams);
        return new DefaultMethod(method);
    }

    private void validate(Method real, Method edge) {
        Class realReturn = real.getReturnType();
        Class edgeReturn = edge.getReturnType();
        validate(edge, realReturn, edgeReturn);
    }

    private void validate(Method method, Class real, Class edge) {
        Class unedged = unedger.unedge(edge);
        if (edge == void.class) return;
        if (real == unedged) return;
        // FIX 3191 Feb 24, 2009 Error message should use "edge", not "unedged".  Surely.
        // FIX 3191 Feb 25, 2009 Should create an explicit MethodWarpException, or some such.
        throw new IllegalArgumentException("Edge method \"" + method +
                "\" has invalid return type, must be (real or edge) variant of \""
                + real + "\".  Instead it is \"" + unedged + "\".");
    }
}
