package au.net.netstorm.boost.bullet.time.core;

import au.net.netstorm.boost.sniper.core.LifecycleTestCase;
import au.net.netstorm.boost.sniper.marker.HasFixtures;
import au.net.netstorm.boost.sniper.marker.InjectableTest;
import au.net.netstorm.boost.sniper.marker.LazyFields;
import au.net.netstorm.boost.sniper.reflect.util.FieldTestUtil;

public final class DefaultNowFactoryAtomicTest extends LifecycleTestCase implements HasFixtures, InjectableTest, LazyFields {
    private Long interval = 1000L;
    private TimePoint time = new DefaultTimePoint(2 * interval);
    private TimePoint past = new DefaultTimePoint(interval);
    private TimePoint future = new DefaultTimePoint(3 * interval);
    NowFactory subject;
    FieldTestUtil fielder;
    Clock clockMock;

    public void fixtures() {
        expect.manyCalls(clockMock, time, "now");
        fielder.setInstance(subject, "clock", clockMock);
    }

    public void testNow() {
        TimePoint actual = subject.now();
        assertEquals(time, actual);
    }

    public void testIn() {
        checkInPast(true, past);
        checkInPast(false, future);
        checkInPast(false, time);
        checkInFuture(true, future);
        checkInFuture(false, past);
        checkInFuture(false, time);
    }

    public void testSince() {
        Duration duration = subject.since(past);
        check(duration);
    }

    public void testUntil() {
        Duration duration = subject.until(future);
        check(duration);
    }

    public void testSinceBlowUp() {
        try {
            subject.since(future);
            fail();
        } catch (IllegalArgumentException expected) { }
    }

    public void testUntilBlowUp() {
        try {
            subject.until(past);
            fail();
        } catch (IllegalArgumentException expected) { }
    }

    private void check(Duration duration) {
        long expected = interval;
        long actual = duration.millis;
        assertEquals(expected, actual);
    }

    private void checkInPast(boolean expected, TimePoint time) {
        boolean actual = subject.inPast(time);
        assertEquals(expected, actual);
    }

    private void checkInFuture(boolean expected, TimePoint time) {
        boolean actual = subject.inFuture(time);
        assertEquals(expected, actual);
    }
}
