package au.net.netstorm.boost.bullet.time.core;

import java.util.Date;
import au.net.netstorm.boost.bullet.time.unsure.DefaultTimeRange;

public final class DefaultTimeFactory implements TimeFactory {
    private final TimeRangeMaster master = new DefaultTimeRangeMaster();

    public TimePoint time(Date date) {
        if (date == null) throw new IllegalArgumentException("Cannot create a time from a NULL date.");
        long millis = date.getTime();
        return time(millis);
    }

    public TimePoint time(long millis) {
        return new DefaultTimePoint(millis);
    }

    public TimeRange range(Date startDate, Date endDate) {
        TimePoint start = time(startDate);
        TimePoint end = time(endDate);
        return range(start, end);
    }

    public TimeRange range(long startMillis, long endMillis) {
        TimePoint start = new DefaultTimePoint(startMillis);
        TimePoint end = new DefaultTimePoint(endMillis);
        return range(start, end);
    }

    public TimeRange range(TimePoint start, TimePoint end) {
        StartTime startTime = new StartTime(start);
        EndTime endTime = new EndTime(end);
        Duration duration = master.duration(startTime, endTime);
        return new DefaultTimeRange(startTime, duration);
    }
}
