package au.net.netstorm.boost.bullet.time.core;

import java.util.Date;
import au.net.netstorm.boost.gunge.sledge.java.lang.EdgeSystem;
import au.net.netstorm.boost.sniper.core.LifecycleTestCase;
import au.net.netstorm.boost.sniper.marker.InjectableTest;
import au.net.netstorm.boost.sniper.marker.LazyFields;

public final class DefaultTimePointMasterAtomicTest extends LifecycleTestCase implements InjectableTest, LazyFields {
    TimePointMaster subject;
    EdgeSystem systemMock;
    private static final long MILLIS_1 = 12L;
    private static final long MILLIS_2 = 9L;

    public void testNext() {
        assertEquals(TIME_100, subject.next(TIME_099));
        assertEquals(TIME_101, subject.next(TIME_100));
        assertEquals(TIME_200, subject.next(TIME_199));
        assertEquals(TIME_201, subject.next(TIME_200));
    }

    public void testPrevious() {
        assertEquals(TIME_099, subject.previous(TIME_100));
        assertEquals(TIME_100, subject.previous(TIME_101));
        assertEquals(TIME_199, subject.previous(TIME_200));
        assertEquals(TIME_200, subject.previous(TIME_201));
    }

    public void testGet() {
        checkGet(MILLIS_1);
        checkGet(MILLIS_2);
    }

    public void testPreviousInvalid() {
        try {
            subject.previous(DefaultTimePoint.EPOCH);
            fail();
        } catch (IllegalArgumentException ex) {
        }
    }

    public void testNextInvalid() {
        try {
            subject.next(DefaultTimePoint.ARMAGEDDON);
            fail();
        } catch (IllegalArgumentException ex) {
        }
    }

    private void checkGet(long millis) {
        TimePoint timePoint = subject.get(millis);
        long actual = timePoint.getMillis();
        assertEquals(millis, actual);
    }

    private static final TimeFactory TIME_FACTORY = new DefaultTimeFactory();
    private static final TimePoint TIME_099 = TIME_FACTORY.time(new Date(99));
    private static final TimePoint TIME_100 = TIME_FACTORY.time(new Date(100));
    private static final TimePoint TIME_101 = TIME_FACTORY.time(new Date(101));
    private static final TimePoint TIME_199 = TIME_FACTORY.time(new Date(199));
    private static final TimePoint TIME_200 = TIME_FACTORY.time(new Date(200));
    private static final TimePoint TIME_201 = TIME_FACTORY.time(new Date(201));
}