package au.net.netstorm.boost.demo.spider.collections;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import au.net.netstorm.boost.bullet.incredibles.core.Weaken;
import au.net.netstorm.boost.demo.spider.test.NuSpiderDemooooTest;
import au.net.netstorm.boost.gunge.graph.GraphUtil;
import au.net.netstorm.boost.sniper.marker.InjectableTest;
import au.net.netstorm.boost.sniper.marker.OverlaysWeb;
import au.net.netstorm.boost.spider.api.config.factory.Factorer;
import au.net.netstorm.boost.spider.api.runtime.Nu;
import au.net.netstorm.boost.spider.plugs.factory.supplied.CollectionsFactory;

public final class CollectionsDemoTest extends NuSpiderDemooooTest implements InjectableTest, OverlaysWeb {
    GraphUtil grapher;
    Loadza<Fun> loadza;
    Tunza<Fun> tunza;
    Factorer factorer;
    Weaken weaken;
    Nu nu;

    public void overlay() {
        factorer = spider.resolve(Factorer.class);
        factorer.add(CollectionsFactory.class);
    }

    public void test() {
        check(List.class, new ArrayList());
        check(Map.class, new HashMap());
        check(Set.class, new HashSet());
    }

    public void testCollections() {
        Fun[] funnies = {fun("foo"), fun("bar"), fun("baz")};
        for (Fun f : funnies) tunza.add(f);
        check(funnies, tunza.list());
        check(funnies, tunza.set());
    }

    public void testMap() {
        Fun[] funnies = {fun("x"), fun("y")};
        for (Fun f : funnies) put(f);
        for (Fun f : funnies) check(loadza, f);
    }

    public void testArguments() {
        List one = nu.nu(List.class);
        one.add("x");
        one.add("y");
        List two = nu.nu(List.class, one);
        assertEquals(one, two);
    }

    private void check(Class cls, Object expected) {
        assertEquals(expected, nu(cls));
        assertEquals(expected, resolve(cls));
    }

    private void check(Fun[] funnies, Collection<Fun> collection) {
        Fun[] actual = collection.toArray(new Fun[3]);
        assertEqualsUnordered(funnies, actual);
    }

    private void check(Loadza<Fun> loadza, Fun expected) {
        String k = weaken.w(expected);
        Fun actual = loadza.get(k);
        assertEquals(expected, actual);
    }

    private Object resolve(Class type) {
        return spider.resolve(type);
    }

    private Object nu(Class type) {
        return spider.nu(type);
    }

    private Fun fun(String v) {
        return nu.nu(Fun.class, v);
    }

    private void put(Fun f) {
        String k = weaken.w(f);
        loadza.put(k, f);
    }
}
