package au.net.netstorm.boost.demo.spider.trapdoor.blueprint;

import au.net.netstorm.boost.gunge.graph.GraphUtil;
import au.net.netstorm.boost.sniper.core.LifecycleTestCase;
import au.net.netstorm.boost.sniper.marker.HasFixtures;
import au.net.netstorm.boost.sniper.marker.InjectableTest;
import au.net.netstorm.boost.spider.api.runtime.Resolver;

public final class BlueprintDemoTest extends LifecycleTestCase implements HasFixtures, InjectableTest {
    Resolver resolver;
    GraphUtil grapher;

    public void fixtures() {
        wire.cls(Weipa.class).to(Town.class);
        wire.cls(FnqFlyingDoctor.class).one().to(FlyingDoctor.class);
        wire.cls(BrisbaneAirField.class).to(AirField.class);
        wire.cls(CoastalFlyingDoctor.class).one().to(FlyingDoctor.class, BrisbaneAirField.class);
        wire.cls(FlyingAmbulance.class).one().to(FlyingDoctor.class, BrisbaneAirField.class, "ambulance");
    }

    public void testSingle() {
        Town t1 = resolveTown();
        Town t2 = resolveTown();
        checkDifferent(t1, t2);
        FlyingDoctor d1 = getDoctor(t1);
        FlyingDoctor d2 = getDoctor(t2);
        checkSame(d1, d2);
    }

    public void testHostedSingle() {
        AirField airField = resolver.resolve(AirField.class);
        check(airField, "doctor", CoastalFlyingDoctor.class);
        check(airField, "ambulance", FlyingAmbulance.class);
    }

    private Town resolveTown() {
        return resolver.resolve(Town.class);
    }

    private void check(AirField airField, String fieldName, Class expected) {
        FlyingDoctor doctor = getDoctor(airField, fieldName);
        Class actual = doctor.getClass();
        assertEquals(expected, actual);
    }

    private FlyingDoctor getDoctor(Object ref) {
        return getDoctor(ref, "doctor");
    }

    private FlyingDoctor getDoctor(Object ref, String fieldName) {
        return (FlyingDoctor) grapher.get(ref, fieldName);
    }

    private void checkSame(Object o1, Object o2) {
        assertNotNull(o1);
        assertEquals(true, o1 == o2);
    }

    private void checkDifferent(Object o1, Object o2) {
        assertNotNull(o1);
        assertNotNull(o2);
        assertEquals(true, o1 != o2);
    }
}
