package au.net.netstorm.boost.demo.spider.trapdoor.resolve;

import au.net.netstorm.boost.sniper.core.LifecycleTestCase;
import au.net.netstorm.boost.sniper.marker.HasFixtures;
import au.net.netstorm.boost.sniper.marker.InjectableTest;
import au.net.netstorm.boost.spider.api.runtime.Impl;
import au.net.netstorm.boost.spider.api.runtime.Resolver;

// SUGGEST: This test could and should be a lot simpler.  Check out BlueprintDemoTest.
public final class ResolveDemoTest extends LifecycleTestCase implements HasFixtures, InjectableTest {
    private Actor peter = new PeterSellers();
    Resolver resolver;
    Impl impl;

    public void fixtures() {
        wire.cls(JeffBridges.class).to(TheDude.class);
        wire.cls(ClassicQuote.class).to(Quote.class);
        wire.cls(BigLebowski.class).to(Movie.class);
        wire.cls(RegalCinema.class).to(Cinema.class);
        wire.ref(peter).to(Actor.class);
        wire.cls(BritneySpears.class).one().to(Celebrity.class);
        wire.cls(MovieBusiness.class).to(Business.class);
        wire.cls(GlitzyHollywood.class).to(Hollywood.class);
    }

    public void testNoArgProvide() {
        TheDude theDude = impl.impl(JeffBridges.class);
        checkTheDudeIsReallyJeff(theDude);
    }

    public void testProvide() {
        Cinema regalCinema = impl.impl(RegalCinema.class);
        assertNotNull(regalCinema);
    }

    public void testProvideSingleton() {
        Hollywood hollywood = nu(GlitzyHollywood.class);
        Business business = nu(MovieBusiness.class);
        checkSameInternals(hollywood, business);
    }

    private void checkSameInternals(Hollywood hollywood, Business business) {
        checkSameActor(hollywood, business);
        checkSameCelebrity(hollywood, business);
    }

    private void checkSameCelebrity(Hollywood hollywood, Business business) {
        Celebrity hollywoodCelebrity = hollywood.getCelebrity();
        Celebrity movieCelebrity = business.getCelebrity();
        assertSame(hollywoodCelebrity, movieCelebrity);
    }

    private void checkSameActor(Hollywood hollywood, Business business) {
        Actor hollywoodActor = hollywood.getActor();
        Actor movieActor = business.getActor();
        assertNotNull(movieActor);
        assertSame(hollywoodActor, movieActor);
        assertEquals(peter, hollywoodActor);
    }

    private void checkTheDudeIsReallyJeff(TheDude theDude) {
        assertNotNull(theDude);
        assertEquals(true, theDude instanceof JeffBridges);
        Quote quote = theDude.getQuote();
        assertNotNull(quote);
    }

    private <T> T nu(Class<? extends T> type) {
        T ref = impl.impl(type);
        assertNotNull(ref);
        return ref;
    }
}
