package au.net.netstorm.boost.gunge.exception;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.UndeclaredThrowableException;
import au.net.netstorm.boost.gunge.sledge.support.SledgeException;
import au.net.netstorm.boost.sniper.core.LifecycleTestCase;
import au.net.netstorm.boost.sniper.marker.InjectableTest;
import au.net.netstorm.boost.sniper.marker.LazyFields;

// DEBT ClassDataAbstractionCoupling {
public final class DefaultThrowableMasterModuleTest extends LifecycleTestCase implements LazyFields, InjectableTest {
    ThrowableMaster subject;
    String trace;

    public void testChecked() {
        isChecked(true, EXCEPTION_1);
        isChecked(true, EXCEPTION_2);
        isChecked(true, THROWABLE_1);
        isChecked(true, THROWABLE_2);
        isChecked(false, RUNTIME_1);
        isChecked(false, RUNTIME_2);
        isChecked(false, ERROR_1);
        isChecked(false, ERROR_2);
    }

    public void testContains() {
        checkContains(true, TIER_1, Tier1Exception.class);
        checkContains(true, TIER_2, Tier1Exception.class, Tier2Exception.class);
        checkContains(true, TIER_3, Tier1Exception.class, Tier2Exception.class, Tier3Exception.class);
        checkContains(true, TIER_NULL, Tier2Exception.class);
        checkContains(false, TIER_1, Exception.class, Error.class, Throwable.class, Tier2Exception.class);
        checkContains(false, TIER_NULL, Exception.class);
        checkContains(false, EXCEPTION_1, Throwable.class);
    }

    public void testRethrow() {
        checkThrowsWrapped(EXCEPTION_1);
        checkThrowsOriginal(RUNTIME_1);
        checkThrowsOriginal(ERROR_2);
    }

    public void testTrace() {
        TestThrowable throwable = new TestThrowable(trace);
        String actualTrace = subject.trace(throwable);
        assertEquals(trace, actualTrace);
    }

    public void testRootCause() {
        Throwable ise = new IllegalStateException();
        Throwable ute = new UndeclaredThrowableException(ise);
        Throwable ite = new InvocationTargetException(ute);
        checkRoot(ise, ise);
        checkRoot(ise, ute);
        checkRoot(ise, ite);
    }

    public void testRealCause() {
        Throwable expected = new Exception();
        Throwable ite = makeInvocationTarget(expected);
        Throwable actual = subject.realCause(ite);
        assertEquals(expected, actual);
        actual = subject.realCause(expected);
        assertEquals(expected, actual);
    }

    private Throwable makeInvocationTarget(Throwable expected) {
        Throwable ute = new UndeclaredThrowableException(expected);
        InvocationTargetException ite = new InvocationTargetException(ute);
        return new SledgeException(ite);
    }

    private void checkRoot(Throwable expected, Throwable actual) {
        Throwable cause = subject.rootCause(actual);
        assertEquals(expected, cause);
    }

    private void checkContains(boolean expected, Throwable t, Class... classes) {
        for (Class cls : classes) checkContains(expected, t, cls);
    }

    private void checkContains(boolean expected, Throwable t, Class cls) {
        boolean actual = subject.contains(t, cls);
        assertEquals(expected, actual);
    }

    private void isChecked(boolean expected, Throwable throwable) {
        boolean result = subject.checked(throwable);
        assertEquals(expected, result);
    }

    private void checkThrowsOriginal(Throwable t) {
        try {
            subject.rethrow(t);
            fail();
        } catch (Throwable caught) {
            assertSame(t, caught);
        }
    }

    private void checkThrowsWrapped(Throwable exception) {
        try {
            subject.rethrow(exception);
        } catch (UndeclaredThrowableException caught) {
            Throwable actual = caught.getUndeclaredThrowable();
            assertSame(exception, actual);
        }
    }

    private static final Throwable EXCEPTION_1 = new Exception();
    private static final Throwable EXCEPTION_2 = new ClassNotFoundException();
    private static final Throwable TIER_1 = new Tier1Exception();
    private static final Throwable TIER_2 = new Tier2Exception(TIER_1);
    private static final Throwable TIER_3 = new Tier3Exception(TIER_2);
    private static final Throwable TIER_NULL = new Tier2Exception(null);
    private static final Throwable THROWABLE_1 = new Throwable();
    private static final Throwable THROWABLE_2 = new DirectThrowableException();
    private static final Throwable RUNTIME_1 = new RuntimeException();
    private static final Throwable RUNTIME_2 = new IllegalArgumentException();
    private static final Throwable ERROR_1 = new Error();
    private static final Throwable ERROR_2 = new InternalError();
}
// } DEBT ClassDataAbstractionCoupling
