package au.net.netstorm.boost.gunge.generics;

import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;

public final class DefaultInterfaceLocator implements InterfaceLocator {
    Generics generics;

    public Type find(Class marker, Class target) {
        Type[] ifaces = target.getGenericInterfaces();
        for (Type type : ifaces) {
            if (match(marker, type)) return type;
            if (compatible(marker, type)) return recurse(marker, type);
        }
        throw new IllegalArgumentException("Could not locate marker [" + marker + "] in heirachy of " + target);
    }

    private Type recurse(Class marker, Type type) {
        Class cls = generics.erased(type);
        return find(marker, cls);
    }

    private boolean match(Class marker, Type iface) {
        return iface instanceof ParameterizedType && marked(marker, iface);
    }

    private boolean marked(Class marker, Type iface) {
        ParameterizedType type = (ParameterizedType) iface;
        return type.getRawType() == marker;
    }

    private boolean compatible(Class marker, Type type) {
        Class cls = generics.erased(type);
        return marker.isAssignableFrom(cls);
    }
}
