package au.net.netstorm.boost.gunge.goop;

import au.net.netstorm.boost.gunge.type.Immutable;
import au.net.netstorm.boost.sniper.core.BoooostCase;
import au.net.netstorm.boost.sniper.reflect.checker.ClassTestChecker;
import au.net.netstorm.boost.sniper.reflect.checker.DefaultClassTestChecker;

public class BootstrapNvpAtomicTest extends BoooostCase {
    private static final String NAME_1 = "x";
    private static final String NAME_2 = "y";
    private static final Object VALUE_1 = new Object();
    private static final Object VALUE_2 = new Object();
    private static final Nvp NVP_1 = new BootstrapNvp("field1", 77);
    private static final Nvp NVP_2 = new BootstrapNvp("field7", "Is absolute zero cold enough?");
    private static final Nvp NVP_NULL_1 = new BootstrapNvp(NAME_1, null);
    private static final Nvp NVP_NULL_2 = new BootstrapNvp("field1", null);
    private static final Object VALUE_CAN_BE_NULL = null;
    private final ClassTestChecker classer = new DefaultClassTestChecker();

    public void testMarker() {
        classer.checkImplementsAndFinal(Immutable.class, BootstrapNvp.class);
    }

    public void testNullNameIllegalInConstructor() {
        checkNullNameIllegalInConstructor(null, VALUE_1);
    }

    public void testCreate() {
        checkCreate(NAME_1, VALUE_CAN_BE_NULL);
        checkCreate(NAME_1, VALUE_1);
        checkCreate(NAME_2, VALUE_2);
    }

    public void testHashCode() {
        checkHashCode("field1", NVP_1);
        checkHashCode("field7", NVP_2);
    }

    public void testEquals() {
        checkDifferentClasses();
        checkNames();
        checkSimpleValues();
    }

    public void testToString() {
        checkToString("Nvp[name=field1,value=77]", NVP_1);
        checkToString("Nvp[name=field7,value=Is absolute zero cold enough?]", NVP_2);
    }

    private void checkDifferentClasses() {
        checkNotEquals(NVP_1, VALUE_1);
        checkNotEquals(VALUE_1, NVP_1);
    }

    private void checkNames() {
        checkEquals(NVP_1, NVP_1);
        checkEquals(NVP_1, NVP_1);
        checkNotEquals(NVP_1, NVP_2);
        checkNotEquals(NVP_2, NVP_1);
    }

    private void checkSimpleValues() {
        checkEquals(NVP_1, NVP_1);
        checkEquals(NVP_NULL_1, NVP_NULL_1);
        checkNotEquals(NVP_1, NVP_NULL_1);
        checkNotEquals(NVP_1, NVP_NULL_2);
        checkNotEquals(NVP_NULL_1, NVP_1);
    }

    private void checkHashCode(String s, Nvp nvp) {
        int expected = s.hashCode();
        int actual = nvp.hashCode();
        assertEquals(expected, actual);
    }

    private void checkToString(String expected, Nvp spec) {
        assertEquals(expected, spec.toString());
    }

    private void checkCreate(String name, Object value) {
        Nvp fieldValue = new BootstrapNvp(name, value);
        assertEquals(name, fieldValue.name());
        assertEquals(value, fieldValue.value());
    }

    private void checkNullNameIllegalInConstructor(String name, Object value) {
        try {
            new BootstrapNvp(name, value);
            fail();
        } catch (IllegalArgumentException expected) {}
    }

    private void checkEquals(Object o1, Object o2) {
        assertEquals(o1, o2);
    }

    private void checkNotEquals(Object o1, Object o2) {
        assertEquals(false, o1.equals(o2));
    }
}
