package au.net.netstorm.boost.gunge.reflect.cls;

import java.lang.reflect.Proxy;
import java.rmi.Remote;
import java.security.PrivateKey;
import java.util.Collection;
import au.net.netstorm.boost.gunge.type.DefaultInterface;
import au.net.netstorm.boost.gunge.type.Interface;
import au.net.netstorm.boost.sniper.core.BoooostCase;

public class DefaultClassNamesAtomicTest extends BoooostCase {
    private final ClassNames names = new DefaultClassNames();

    public void testGetShortNameByClass() {
        checkGetShortNameByClass("Object", Object.class);
        checkGetShortNameByClass("Proxy", Proxy.class);
        checkGetShortNameByClass("String", String.class);
    }

    public void testGetShortNameByInterface() {
        testGetShortNameByInterface("Remote", Remote.class);
        testGetShortNameByInterface("CharSequence", CharSequence.class);
    }

    public void testGetPackageNameByClass() {
        checkGetPackageNameByClass("java.lang", String.class);
        checkGetPackageNameByClass("java.lang.reflect", Proxy.class);
    }

    public void testGetPackageNameByInterface() {
        checkGetPackageNameByInterface("java.util", Collection.class);
        checkGetPackageNameByInterface("java.security", PrivateKey.class);
    }

    private void checkGetShortNameByClass(String expected, Class cls) {
        String shortName = names.shorty(cls);
        assertEquals(expected, shortName);
    }

    private void testGetShortNameByInterface(String expected, Class cls) {
        Interface iface = new DefaultInterface(cls);
        String shortName = names.shorty(iface);
        assertEquals(expected, shortName);
    }

    private void checkGetPackageNameByClass(String expected, Class cls) {
        String packageName = names.pkg(cls);
        assertEquals(expected, packageName);
    }

    private void checkGetPackageNameByInterface(String expected, Class cls) {
        Interface iface = new DefaultInterface(cls);
        String packageName = names.pkg(iface);
        assertEquals(expected, packageName);
    }
}
