package au.net.netstorm.boost.gunge.set;

// FIX 1202 Jan 15, 2009 Reimplement me. And give me a lovely new home.
public final class HashSet<T> implements Set<T> {
    final Object empty = new Object();
    final int notfound = -1;
    final Evictor evictor;
    final Link link;
    final double limit;
    int count = 0;
    Object[] set;

    public HashSet(Link link, Evictor evictor, int initial, double limit) {
        this.link = link;
        this.evictor = evictor;
        this.set = new Object[initial];
        this.limit = limit;
    }

    public void put(T t) {
        if (has(t)) throw new IllegalArgumentException("no dupes");
        sweep();
        ++count;
        resize();
        put(set, t);
    }

    public void remove(T t) {
        int b = find(t);
        if (b == notfound) throw new IllegalArgumentException("element not found");
        kill(b);
    }

    public boolean has(T t) {
        int b = find(t);
        return b != notfound;
    }

    // DEBT NCSS {
    private int find(T t) {
        int b = optimal(set, t);
        int start = b;
        while (set[b] != null) {
            if (link.equal(set[b], t)) return b;
            b = next(b, set);
            if (b == start) break;
        }
        return notfound;
    }
    // } DEBT NCSS

    private void sweep() {
        for (int i = 0; i < set.length; ++i) {
            if (evictor.evict(set[i])) kill(i);
        }
    }

    private void kill(int b) {
        --count;
        set[b] = empty;
    }

    private void put(Object[] set, Object o) {
        int b = bucket(set, o);
        set[b] = o;
    }

    private int bucket(Object[] set, Object o) {
        int optimal = optimal(set, o);
        return free(set, optimal);
    }

    private int free(Object[] set, int b) {
        int result = b;
        while (!empty(set[result])) result = next(result, set);
        return result;
    }

    private void resize() {
        float load = (float) count / set.length;
        if (load < limit) return;
        grow();
    }

    private void grow() {
        Object[] nu = new Object[set.length * 2];
        rehash(set, nu);
        set = nu;
    }

    private void rehash(Object[] from, Object[] to) {
        for (Object ref : from) rehash(to, ref);
    }

    private void rehash(Object[] target, Object ref) {
        if (empty(ref)) return;
        put(target, ref);
    }

    private int optimal(Object[] target, Object ref) {
        int h = link.hash(ref);
        return h % target.length;
    }

    private int next(int i, Object[] s) {
        return (i + 1) % s.length;
    }

    private boolean empty(Object ref) {
        return ref == null || ref == empty;
    }
}
