package au.net.netstorm.boost.sniper.reflect.util;

import java.lang.reflect.Constructor;
import au.net.netstorm.boost.gunge.reflect.constructor.DefaultSingleConstructor;
import au.net.netstorm.boost.gunge.reflect.constructor.SingleConstructor;
import au.net.netstorm.boost.gunge.sledge.java.lang.reflect.DefaultEdgeConstructor;
import au.net.netstorm.boost.gunge.sledge.java.lang.reflect.EdgeConstructor;
import au.net.netstorm.boost.gunge.type.Interface;

public class DefaultClassTestUtil implements ClassTestUtil {
    private final SingleConstructor reflector = new DefaultSingleConstructor();
    private final EdgeConstructor edgeConstructor = new DefaultEdgeConstructor();
    private static final Object[] NO_PARAMETERS = {};

    // FIX SC509 Reorder parameters?
    public boolean isImplementationOf(Interface targetInterface, Class cls) {
        Class type = targetInterface.v();
        return isAssignable(type, cls);
    }

    public boolean isSubclassOf(Class superClass, Class subClass) {
        return isAssignable(superClass, subClass);
    }

    public boolean isSubInterfaceOf(Interface superInterface, Interface subInterface) {
        Class superType = superInterface.v();
        Class subType = subInterface.v();
        return isAssignable(superType, subType);
    }

    public Object newInstance(Class type) {
        Constructor constructor = reflector.ctor(type);
        constructor.setAccessible(true);
        return edgeConstructor.newInstance(constructor, NO_PARAMETERS);
    }

    public boolean isInstantiable(Class<?> type) {
        Constructor[] ctors = type.getDeclaredConstructors();
        if (ctors.length != 1) return false;
        Class<?>[] signature = ctors[0].getParameterTypes();
        return signature.length == 0;
    }

    private boolean isAssignable(Class superType, Class subType) {
        return superType.isAssignableFrom(subType);
    }
}
