package au.net.netstorm.boost.spider.api.builder;

import au.net.netstorm.boost.gunge.goop.Nvp;
import au.net.netstorm.boost.gunge.reflect.field.DefaultFieldNvps;
import au.net.netstorm.boost.gunge.reflect.field.FieldNvps;
import au.net.netstorm.boost.gunge.sledge.java.lang.DefaultEdgeClass;
import au.net.netstorm.boost.gunge.sledge.java.lang.EdgeClass;
import au.net.netstorm.boost.spider.api.config.scope.Scoper;
import au.net.netstorm.boost.spider.api.config.web.Web;
import au.net.netstorm.boost.spider.api.runtime.Injector;

public class DefaultSpinneret implements Spinneret {
    private final FieldNvps fielder = new DefaultFieldNvps();
    private final EdgeClass classer = new DefaultEdgeClass();
    Injector injector;
    Scoper scoper;

    // FIX 3324 Mar 13, 2009 Delete.
    public String toString() {
        return "injector = " + injector + " scoper = " + scoper;
    }

    public void spin(Class<? extends Web>... webs) {
        for (Class<? extends Web> web : webs) spin(web);
    }

    public void spin(Class<? extends Web> web) {
        Web instance = classer.newInstance(web);
        scope(instance);
        injector.inject(instance);
        instance.web();
    }

    private void scope(Web web) {
        Nvp[] nvps = fielder.nvps(web);
        for (Nvp nvp : nvps) scope(nvp);
    }

    private void scope(Nvp nvp) {
        String name = nvp.name();
        Object value = nvp.value();
        scope(name, value);
    }

    private void scope(String name, Object value) {
        if (!name.equals("scope")) return;
        if (value == null) barf("Web scope field can not be null.");
        scope(value);
    }

    private void scope(Object value) {
        if (value instanceof String) single(value);
        else if (value instanceof String[]) multi(value);
        else barf("Web scope field has invalid type, must be String or String[]");
    }

    private void single(Object value) {
        String scope = (String) value;
        scoper.scope(scope);
    }

    private void multi(Object value) {
        String[] scopes = (String[]) value;
        for (String scope : scopes) scoper.scope(scope);
    }

    private void barf(String msg) {
        throw new IllegalArgumentException(msg);
    }
}
