package au.net.netstorm.boost.spider.core.injection.types;

import java.util.Arrays;
import au.net.netstorm.boost.bullet.primordial.Primordial;
import au.net.netstorm.boost.gunge.equals.DefaultEqualsConstraints;
import au.net.netstorm.boost.gunge.equals.EqualsConstraints;

public class DefaultInjectionType<T> extends Primordial implements InjectionType {
    private final Class<T> raw;
    private final Class[] params;

    public DefaultInjectionType(Class<T> raw, Class... params) {
        if (raw == null) throw new IllegalArgumentException();
        this.params = params;
        this.raw = raw;
    }

    public InjectionType<T> raw() {
        return new DefaultInjectionType<T>(raw);
    }

    public Class[] parameters() {
        return params;
    }

    public Class<T> rawClass() {
        return raw;
    }

    public boolean isInterface() {
        return raw.isInterface();
    }

    public boolean supportsResolution() {
        return true;
    }

    public boolean equals(Object o) {
        if (!constraints.ok(this, o)) return false;
        InjectionType other = (InjectionType) o;
        if (raw != other.rawClass()) return false;
        return Arrays.deepEquals(params, other.parameters());
    }

    public int hashCode() {
        return raw.hashCode() + Arrays.hashCode(params);
    }

    private static final EqualsConstraints constraints = new DefaultEqualsConstraints();
}
