package au.net.netstorm.boost.spider.internals.bindings.constraints;

import au.net.netstorm.boost.gunge.optional.Optional;
import au.net.netstorm.boost.spider.core.binding.Pattern;

// DEBT CyclomaticComplexity {
public class DefaultConstrainer implements Constrainer {
    public BindingConstraint constrain(Pattern pattern) {
        Optional<Class> host = pattern.host();
        Optional<String> name = pattern.name();
        return decide(host, name);
    }

    private BindingConstraint decide(Optional<Class> host, Optional<String> name) {
        if (host.isSet() && name.isSet()) return both(host, name);
        if (host.isSet()) return hosted(host);
        if (name.isSet()) return named(name);
        return any();
    }

    private BindingConstraint any() {
        return new AnyBindingConstraint();
    }

    private BindingConstraint hosted(Optional<Class> optional) {
        Class host = optional.get();
        return new HostedBindingConstraint(host);
    }

    private BindingConstraint named(Optional<String> optional) {
        String name = optional.get();
        return new NamedBindingConstraint(name);
    }

    private BindingConstraint both(Optional<Class> host, Optional<String> name) {
        BindingConstraint hosted = hosted(host);
        BindingConstraint named = named(name);
        return new HostedAndNamedBindingConstraint(hosted, named);
    }
}
// } DEBT CyclomaticComplexity
