package au.net.netstorm.boost.spider.internals.bindings.core;

import au.net.netstorm.boost.gunge.optional.NotSetOptional;
import au.net.netstorm.boost.gunge.optional.Optional;
import au.net.netstorm.boost.gunge.optional.SetOptional;
import au.net.netstorm.boost.spider.core.binding.Binding;
import au.net.netstorm.boost.spider.core.binding.Pattern;
import au.net.netstorm.boost.spider.core.injection.sites.InjectionSite;
import au.net.netstorm.boost.spider.core.injection.types.InjectionType;
import au.net.netstorm.boost.spider.internals.bindings.constraints.BindingConstraint;
import au.net.netstorm.boost.spider.internals.bindings.constraints.Constrainer;

public final class DefaultBindingsFu implements BindingsFu {
    Constrainer constrainer;
    BindingsGu gu;

    public Optional<Binding> lookup(InjectionSite site) {
        InjectionType type = site.type();
        Optional<Binding> optional = lookup(site, type);
        if (!optional.isSet()) optional = erased(site, type);
        return optional;
    }

    public void add(Binding binding) {
        gu.add(binding);
    }

    private Optional<Binding> erased(InjectionSite site, InjectionType type) {
        InjectionType erased = type.raw();
        return lookup(site, erased);
    }

    private Optional<Binding> lookup(InjectionSite site, InjectionType type) {
        for (Binding binding : gu.lookup(type)) {
            if (accepts(binding, site, type)) return new SetOptional<Binding>(binding);
        }
        return new NotSetOptional<Binding>();
    }

    private boolean accepts(Binding binding, InjectionSite site, InjectionType type) {
        Pattern pattern = binding.pattern();
        BindingConstraint constraint = constrainer.constrain(pattern);
        // FIX 2394 Should this check allow for variance? or erasure?
        InjectionType actual = pattern.type();
        return constraint.accept(site) && actual.equals(type);
    }
}
