package au.net.netstorm.boost.spider.internals.bootstrap.build;

import au.net.netstorm.boost.spider.api.builder.Spinneret;
import au.net.netstorm.boost.spider.api.config.scope.Scoper;
import au.net.netstorm.boost.spider.api.config.wire.DefaultLine;
import au.net.netstorm.boost.spider.api.config.wire.Line;
import au.net.netstorm.boost.spider.api.runtime.Spider;
import au.net.netstorm.boost.spider.internals.bindings.resolver.DefaultWiringConnectrex;
import au.net.netstorm.boost.spider.internals.bootstrap.mini.DefaultMiniRefs;
import au.net.netstorm.boost.spider.internals.bootstrap.mini.DefaultMiniRegistry;
import au.net.netstorm.boost.spider.internals.bootstrap.mini.DefaultMiniSpider;
import au.net.netstorm.boost.spider.internals.bootstrap.mini.MiniRefs;
import au.net.netstorm.boost.spider.internals.bootstrap.mini.MiniRegistry;
import au.net.netstorm.boost.spider.internals.bootstrap.mini.MiniSpider;
import au.net.netstorm.boost.spider.internals.entry.DefaultGrapherFu;
import au.net.netstorm.boost.spider.internals.entry.ErrorGrapherFu;
import au.net.netstorm.boost.spider.internals.entry.GrapherFu;
import au.net.netstorm.boost.spider.internals.errors.ErrorInstantiationProbe;
import au.net.netstorm.boost.spider.internals.graph.decorator.AspectDecorator;
import au.net.netstorm.boost.spider.internals.graph.decorator.Decorator;
import au.net.netstorm.boost.spider.internals.probe.HubProbe;
import au.net.netstorm.boost.spider.internals.probe.Probe;
import au.net.netstorm.boost.spider.internals.probe.Prober;
import au.net.netstorm.boost.spider.plugs.connectrex.core.Connectrex;
import au.net.netstorm.boost.spider.plugs.connectrex.core.DefaultConnectrexes;
import au.net.netstorm.boost.spider.plugs.provider.core.Provider;

// OK ClassDataAbstractionCoupling|NCSS|LineLength {
public class SpiderBootstrapper implements Bootstrapper {
    MiniRefs refs = new DefaultMiniRefs();
    MiniRegistry registry = new DefaultMiniRegistry();

    {
        // Nuggets of yukiness?
        registry.add(Decorator.class, AspectDecorator.class);
        registry.add(GrapherFu.class, DefaultGrapherFu.class, ErrorGrapherFu.class);
        registry.add(GrapherFu.class, ErrorGrapherFu.class);
        registry.add(Probe.class, HubProbe.class);
        //
        registry.add(Provider.class, BootstrapProvider.class);
        registry.add(Connectrex.class, BootstrapConnectrex.class, DefaultConnectrexes.class, "bootstrap");
        registry.add(Connectrex.class, DefaultWiringConnectrex.class,  DefaultConnectrexes.class,  "wiring");
    }

    MiniSpider mini = new DefaultMiniSpider(refs, registry);
    /* circular holder */
    Spider spider = mini.resolve(Spider.class);

    {
        FutureSpider future = refs.get(FutureSpider.class);
        future.set(spider);
    }

    public Spider bootstrap() {
        logging();
        wiring();
        web();
        return spider;
    }

    private void logging() {
        // FIX 3324 Mar 11, 2009 Plug error probe in here.
        ErrorInstantiationProbe errors = refs.get(ErrorInstantiationProbe.class);
        Prober prober = mini.resolve(Prober.class);
        prober.probe(errors);
    }

    private void wiring() {
        BootstrapWire wire = mini.resolve(BootstrapWire.class);
        wire.impl(DefaultLine.class, Line.class);
    }

    private void web() {
        Scoper scoper = mini.resolve(Scoper.class);
        scoper.scope("au.net.netstorm.boost.spider");
        Spinneret spinneret = mini.resolve(Spinneret.class);
        spinneret.spin(SpiderWeb.class);
    }
}
// } OK ClassDataAbstractionCoupling|NCSS|LineLength
