package au.net.netstorm.boost.spider.internals.bootstrap.mini;

import java.util.ArrayList;
import java.util.List;
import au.net.netstorm.boost.gunge.type.DefaultInterface;
import au.net.netstorm.boost.gunge.type.Interface;
import au.net.netstorm.boost.spider.internals.graph.scope.PrefixScoping;
import au.net.netstorm.boost.spider.internals.graph.scope.Scoping;

public class DefaultMiniInterfacer implements MiniInterfacer {
    // Only necessary to deal with cobertura's pollution of our source code
    // for coverage checks, specifically interface HasBeenInstrumented.
    final Scoping scoping = new PrefixScoping("au.net.netstorm.boost");

    public Interface iface(Object ref) {
        Class impl = ref.getClass();
        List<Class> faces = ifaces(impl);
        return iface(impl, faces);
    }

    private List<Class> ifaces(Class impl) {
        Class[] ifaces = impl.getInterfaces();
        return scope(ifaces);
    }

    private List<Class> scope(Class[] ifaces) {
        List<Class> scoped = new ArrayList<Class>();
        for (Class iface : ifaces)
            if (scoping.in(iface))
                scoped.add(iface);
        return scoped;
    }

    private Interface iface(Class impl, List<Class> faces) {
        if (faces.size() != 1) barf(impl, faces);
        return new DefaultInterface(faces.get(0));
    }

    private void barf(Class impl, List<Class> ifaces) {
        String msg = "Mini only deals with classes which implement one boost super interface";
        throw new IllegalArgumentException(msg + " - " + impl + " " + stringify(ifaces));
    }

    private String stringify(List<Class> faces) {
        String r = "interfaces(";
        for (Class face : faces)
            r += " " + face;
        return r + " )";
    }
}
