package au.net.netstorm.boost.spider.internals.graph.core;

import java.util.LinkedList;
import java.util.List;
import au.net.netstorm.boost.spider.api.bang.UnresolvableException;
import au.net.netstorm.boost.spider.core.binding.Binding;
import au.net.netstorm.boost.spider.core.injection.sites.InjectionSite;
import au.net.netstorm.boost.spider.plugs.provider.core.Provider;

public class DefaultNode implements Node {
    private final List<Node> children = new LinkedList<Node>();
    private final List<Node> dependencies = new LinkedList<Node>();
    private final InjectionSite site;
    private Provider provider;
    private Binding binding;

    public DefaultNode(InjectionSite site) {
        this.site = site;
    }

    public void addChild(Node child) {
        children.add(child);
    }

    public void addDependency(Node dependency) {
        dependencies.add(dependency);
    }

    public List<Node> children() {
        return children;
    }

    public List<Node> dependencies() {
        return dependencies;
    }

    public InjectionSite site() {
        return site;
    }

    public Provider provider() {
        if (!hasProvider()) throw new UnresolvableException(site);
        return provider;
    }

    public void setProvider(Provider provider) {
        this.provider = provider;
    }

    public boolean hasProvider() {
        return provider != null;
    }

    public Binding binding() {
        if (!hasBinding()) throw new UnresolvableException(site);
        return binding;
    }

    public void setBinding(Binding binding) {
        this.binding = binding;
    }

    private boolean hasBinding() {
        return binding != null;
    }
}